// This is basically the PlayRandom example sketch from the DFPlayer_Mini_Mp3_by_Makuna
// library (https://github.com/Makuna/DFMiniMp3/wiki). My only real addition is the manual
// control buttons. F.J. Kraan, fjkraan@electrickery.nl, 2024-01-17

// this example will play a random track from all on the sd 
//
// it expects the sd card to contain some mp3 files

#include <DFMiniMp3.h>

// forward declare the notify class, just the name
//
class Mp3Notify; 

#define DF_RX 10
#define DF_TX 11

// Some arduino boards only have one hardware serial port, so a software serial port is needed instead.
// comment out the above definitions and use these
#include <SoftwareSerial.h>
SoftwareSerial secondarySerial(DF_RX, DF_TX); // RX, TX
typedef DFMiniMp3<SoftwareSerial, Mp3Notify> DfMp3;
DfMp3 dfmp3(secondarySerial);

// implement a notification class,
// its member methods will get called 
//
class Mp3Notify
{
public:
  static void PrintlnSourceAction(DfMp3_PlaySources source, const char* action)
  {
    if (source & DfMp3_PlaySources_Sd) 
    {
        Serial.print("SD Card, ");
    }
    if (source & DfMp3_PlaySources_Usb) 
    {
        Serial.print("USB Disk, ");
    }
    if (source & DfMp3_PlaySources_Flash) 
    {
        Serial.print("Flash, ");
    }
    Serial.println(action);
  }
  static void OnError([[maybe_unused]] DfMp3& mp3, uint16_t errorCode)
  {
    // see DfMp3_Error for code meaning
    Serial.println();
    Serial.print("Com Error ");
    Serial.println(errorCode);
  }
  static void OnPlayFinished([[maybe_unused]] DfMp3& mp3, [[maybe_unused]] DfMp3_PlaySources source, uint16_t track)
  {
    Serial.print("Play finished for #");
    Serial.println(track);  
  }
  static void OnPlaySourceOnline([[maybe_unused]] DfMp3& mp3, DfMp3_PlaySources source)
  {
    PrintlnSourceAction(source, "online");
  }
  static void OnPlaySourceInserted([[maybe_unused]] DfMp3& mp3, DfMp3_PlaySources source)
  {
    PrintlnSourceAction(source, "inserted");
  }
  static void OnPlaySourceRemoved([[maybe_unused]] DfMp3& mp3, DfMp3_PlaySources source)
  {
    PrintlnSourceAction(source, "removed");
  }
};

#define DEBUG 4
bool debug = 0;

#define VOLPLUS 2
#define VOLMIN  3
#define DEFVOL 20;
#define DELAY   500L

uint16_t volume;

#include <EEPROM.h>
#define TFVOLADR 10
#define MAXTFVOL 30

void setup() 
{
  Serial.begin(115200);

  delay(100L);

  Serial.print("VogelSpeler version ");
  Serial.println("0.4n");
  
  Serial.println("initializing...");

  pinMode(VOLPLUS, INPUT_PULLUP);
  pinMode(VOLMIN, INPUT_PULLUP);
  pinMode(DEBUG, INPUT_PULLUP);
  pinMode(LED_BUILTIN, OUTPUT);
  if (digitalRead(DEBUG)) debug = 1;

  dfmp3.begin();

  // during development, it's a good practice to put the module
  // into a known state by calling reset().  
  // You may hear popping when starting and you can remove this 
  // call to reset() once your project is finalized
//  dfmp3.reset(); 

  if (debug) {
      uint16_t version = dfmp3.getSoftwareVersion();
      Serial.print("DFMiniMp3 version ");
      Serial.println(version);
  }

  // show some properties and set the volume
  volume = EEPROM.read(TFVOLADR);
  if (volume > MAXTFVOL) {
    volume = DEFVOL; 
    if (debug) Serial.println("Setting TF volume to default in EEPROM");
    EEPROM.write(TFVOLADR, volume);
  }
  if (debug) Serial.print("volume ");
  if (debug) Serial.println(volume);
  dfmp3.setVolume(volume);

  if (debug) {
    uint16_t count = dfmp3.getTotalTrackCount(DfMp3_PlaySource_Sd);
    Serial.print("files ");
    Serial.println(count);
  
    if (count == 0) {
      Serial.println("No files, halting");
      while(1) {
        blink();
        delay(100L);
      }
    }
  
    uint16_t mode = dfmp3.getPlaybackMode();
    Serial.print("playback mode ");
    Serial.println(mode);
    
    Serial.println("starting...");
  }
  
  dfmp3.playRandomTrackFromAll(); // random of all folders on sd
}

void loop() 
{
   if (!digitalRead(VOLPLUS)) {
    volume++;
    if (volume > 30) volume = 30;
    if (debug) Serial.print("volume: ");
    if (debug) Serial.println(volume);
    dfmp3.setVolume(volume);
    EEPROM.update(TFVOLADR, volume);
    blink();
    delay(DELAY);
  }
  if (!digitalRead(VOLMIN)) {
    volume--;
    if (volume > 30) volume = 0;  // An uint16_t never gets below 0, but wraps to MAXUINT16
    if (debug) Serial.print("volume: ");
    if (debug) Serial.println(volume);
    dfmp3.setVolume(volume);
    EEPROM.update(TFVOLADR, volume);
    blink();
    delay(DELAY);
  }
  // calling dfmp3.loop() periodically allows for notifications 
  // to be handled without interrupts
  dfmp3.loop();
  digitalWrite(LED_BUILTIN, LOW);
}

void blink() {
  digitalWrite(LED_BUILTIN, !digitalRead(LED_BUILTIN));
}
